//     ======================= COM_port.cpp =================================
//     **********************************************************************
//      "   COM " -   ===============
//     Author: Yeremeyev Dmitriy Vladimirovich *** mailto:edvmail@mail.ru
//     Last update : 19.10.2009
//     **********************************************************************
#include "COM_port.h"

//   COM 
TCOMPortDescriptors CPDTab[COMPortDsrTabMaxNum];

//     COM 
void CP_COMPortInit()
{
	//   
	for(int i=0; i<COMPortDsrTabMaxNum; i++){
		CPDTab[i].hCOMPort = 0;
		CPDTab[i].PortName = _T("");
	}
}

//      COM 
void CP_COMPortClose()
{
	for(int i=1; i<COMPortDsrTabMaxNum+1; i++) CP_CloseCOMPort(i);
}

//  COM 
HFILE CP_OpenCOMPort(USHORT aCOMPortNum,int iBaudRate, char cParity,
					 int iByteSize, int iStopBit)
{
	HFILE PIndex;
	wxChar*  COM_InitStr = _T(" COM ");

	if(!aCOMPortNum) return 0;

	wxString PortName = wxString::Format(_T("\\\\.\\COM%u"),aCOMPortNum);
	
	//      
	for(PIndex=0; PIndex<COMPortDsrTabMaxNum && CPDTab[PIndex].hCOMPort; PIndex++);
	//   ?
	if(PIndex>=COMPortDsrTabMaxNum) {// 

		wxMessageBox(wxString::Format(_T("  ,  %s  "),
						PortName), COM_InitStr, wxOK | wxICON_ERROR);
		return 0;
	}
	
	CPDTab[PIndex].hCOMPort = CreateFile(PortName,GENERIC_READ | GENERIC_WRITE, 0,
										NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	//     ?
	if(CPDTab[PIndex].hCOMPort != INVALID_HANDLE_VALUE) { // 
		
		SetupComm(CPDTab[PIndex].hCOMPort, CSRP_BufSize, CSRP_BufSize);
		GetCommTimeouts(CPDTab[PIndex].hCOMPort,&CPDTab[PIndex].TimeOutS);
		CPDTab[PIndex].TimeOutS.ReadIntervalTimeout=MAXDWORD;
		CPDTab[PIndex].TimeOutS.ReadTotalTimeoutMultiplier=0;
		CPDTab[PIndex].TimeOutS.ReadTotalTimeoutConstant=0;
		CPDTab[PIndex].TimeOutS.WriteTotalTimeoutMultiplier=0;
		CPDTab[PIndex].TimeOutS.WriteTotalTimeoutConstant=0;
		if(!SetCommTimeouts(CPDTab[PIndex].hCOMPort,&CPDTab[PIndex].TimeOutS)) {
			CloseHandle(CPDTab[PIndex].hCOMPort);
			CPDTab[PIndex].hCOMPort = 0;
			wxMessageBox(wxString::Format(_T("       %s"),
							PortName), COM_InitStr, wxOK | wxICON_ERROR);
			return 0;
		}
		if(!GetCommState(CPDTab[PIndex].hCOMPort, &CPDTab[PIndex].dcb)){
			CloseHandle(CPDTab[PIndex].hCOMPort);
			CPDTab[PIndex].hCOMPort = 0;

			wxMessageBox(wxString::Format(_T("        %s"),
							PortName), COM_InitStr, wxOK | wxICON_ERROR);
			return 0;
		}
		CPDTab[PIndex].dcb.BaudRate = iBaudRate;
		CPDTab[PIndex].dcb.ByteSize = iByteSize;
		CPDTab[PIndex].dcb.Parity	= cParity;

		switch (iStopBit) {
			case 2:  CPDTab[PIndex].dcb.StopBits = TWOSTOPBITS;break;
			default: CPDTab[PIndex].dcb.StopBits = ONESTOPBIT; break; 
		}
		if (!SetCommState(CPDTab[PIndex].hCOMPort, &CPDTab[PIndex].dcb)) {
			CloseHandle(CPDTab[PIndex].hCOMPort);
			CPDTab[PIndex].hCOMPort = 0;

			wxMessageBox(wxString::Format(_T("        %s"),
							PortName), COM_InitStr, wxOK | wxICON_ERROR);
			return 0;
		}
		CPDTab[PIndex].PortName = PortName;
	} else { // 

		wxMessageBox(wxString::Format(_T("     %s"),
						PortName), COM_InitStr, wxOK | wxICON_ERROR);

		CPDTab[PIndex].hCOMPort = 0;
		return 0;
	}
	return PIndex+1;
}

//   
DWORD CP_ReadCOMPort(HFILE hPort, LPVOID lpBuffer, DWORD nNumberOfBytesToRead)
{
	DWORD lpNumberOfBytesRead;
	
	if(!hPort || hPort>COMPortDsrTabMaxNum) return 0;
	if(!CPDTab[hPort-1].hCOMPort) return 0;
	ReadFile(CPDTab[hPort-1].hCOMPort,lpBuffer,nNumberOfBytesToRead,&lpNumberOfBytesRead,NULL);
	return lpNumberOfBytesRead;
}

//   
DWORD CP_WriteCOMPort(HFILE hPort, LPVOID lpBuffer, DWORD nNumberOfBytesToWrite)
{
	DWORD lpNumberOfBytesWritten;
	
	if(!hPort || hPort>COMPortDsrTabMaxNum) return 0;
	if(!CPDTab[hPort-1].hCOMPort) return 0;
	WriteFile(CPDTab[hPort-1].hCOMPort,lpBuffer,nNumberOfBytesToWrite,&lpNumberOfBytesWritten,NULL);
	return lpNumberOfBytesWritten;
}

//   COM 
wxString* CP_GetCOMPortName(HFILE hPort)
{
	if(!hPort || hPort>COMPortDsrTabMaxNum) return NULL;
	if(!CPDTab[hPort-1].hCOMPort) return NULL;
	return &CPDTab[hPort-1].PortName;
}

//  COM 
void CP_CloseCOMPort(HFILE hPort)
{
	if(!hPort || hPort>COMPortDsrTabMaxNum) return;
	if(!CPDTab[hPort-1].hCOMPort) return;
	
	CloseHandle(CPDTab[hPort-1].hCOMPort);
	if(CPDTab[hPort-1].PortName) {
		CPDTab[hPort-1].PortName = _T("");
	}
	CPDTab[hPort-1].hCOMPort = 0;
}