//     ======================= RobotController.cpp ==========================
//     **********************************************************************
//     Author: Yeremeyev Dmitriy Vladimirovich *** mailto:edvmail@mail.ru
//     Last update : 30.09.2009
//     **********************************************************************

#pragma warning (disable:4996) // This function or variable may be unsafe

#include "RobotController.h"

#define cIndicatorSizeX 230
#define cIndicatorSizeY 310
#define cCmdLoop        15
#define cCmdTotal       10
//   
struct TCmdArray {
	BOOL Cmd[cCmdTotal];
};

HHOOK      hook_keyboard;

TKeyArray* gpKeyArray;

//   
LRESULT CALLBACK LowLevelKeyboardProc(int iCode, WPARAM wParam, LPARAM lParam)
{
 	if (iCode == HC_ACTION) {
		KBDLLHOOKSTRUCT *khs = (KBDLLHOOKSTRUCT*)lParam;

		if(wParam == WM_KEYDOWN || wParam == WM_KEYUP) {
			bool  KeyState = (wParam == WM_KEYDOWN);
			DWORD  KeyCode = khs->vkCode;
			switch(KeyCode) {
				case VK_SPACE:  gpKeyArray->Key[7] = KeyState; break;
				case VK_DELETE: gpKeyArray->Key[6] = KeyState; break;
				case VK_END:    gpKeyArray->Key[5] = KeyState; break;
				case VK_NEXT:   gpKeyArray->Key[4] = KeyState; break;
				case VK_RIGHT:  gpKeyArray->Key[3] = KeyState; break;
				case VK_UP:     gpKeyArray->Key[2] = KeyState; break;
				case VK_DOWN:   gpKeyArray->Key[1] = KeyState; break;
				case VK_LEFT:   gpKeyArray->Key[0] = KeyState; break;
			}
		}
	}

	return CallNextHookEx(hook_keyboard,iCode,wParam,lParam);
}

BEGIN_EVENT_TABLE(CIndicator, wxStaticText)
	EVT_PAINT(CIndicator::OnPaint)
END_EVENT_TABLE()

CIndicator::CIndicator(wxWindow* parent)
       : wxStaticText(parent, wxID_ANY, _T("Indicator"), wxDefaultPosition,
	     wxSize(cIndicatorSizeX, cIndicatorSizeY), wxSUNKEN_BORDER|wxCLIP_CHILDREN)
{
	memset(&mKeyArray, 0, sizeof(TKeyArray));
	mMemBmp = new wxBitmap(cIndicatorSizeX, cIndicatorSizeY);
	mMemDC.SelectObject(*mMemBmp);
}

CIndicator::~CIndicator()
{
	if(mMemBmp) { delete mMemBmp; mMemBmp = NULL; }
}


//   
void CIndicator::OnPaint(wxPaintEvent& WXUNUSED(event))
{
	wxPaintDC dc(this);
	wxRect r = GetClientRect();
	dc.Blit(r.x, r.y, r.GetWidth(), r.GetHeight(), &mMemDC, 0,0);
}

// 
void CIndicator::Draw()
{
	wxPen Pen;
	Pen.SetColour(0x00,0x00,0x00);
	mMemDC.SetPen(Pen);
	wxBrush Br;
	Br.SetColour(0xCF,0xCF,0xCF);
	mMemDC.SetBrush(Br);
	wxRect Rect = GetClientRect();
	mMemDC.DrawRectangle(Rect);

	//    
	Pen.SetColour(0x60,0x60,0x60);
	mMemDC.SetPen(Pen);

	//   
	wxBrush mkBrush;
	mkBrush.SetColour(0xC0,0xC0,0xC0);
	mMemDC.SetBrush(mkBrush);

	//   
	#define cmkX 12
	#define cmkY 120
	#define cmkH 55
	#define cmkN 8
	#define cmbN 4

	static wxPoint mkPts1[cmkN] = {
		wxPoint( cmkX,        cmkY+cmkH   ), wxPoint( cmkX+cmkH, cmkY+cmkH   ),
		wxPoint( cmkX+cmkH,   cmkY        ), wxPoint( cmkX+cmkH*2, cmkY      ),
		wxPoint( cmkX+cmkH*2, cmkY+cmkH   ), wxPoint( cmkX+cmkH*3, cmkY+cmkH ),
		wxPoint( cmkX+cmkH*3, cmkY+cmkH*2 ), wxPoint( cmkX, cmkY+cmkH*2      )
	};

	mMemDC.DrawPolygon(cmkN, mkPts1, 0,0);

	static wxPoint mkPts2[cmbN] = {
		wxPoint( cmkX,        cmkY-cmkH   ), wxPoint( cmkX+cmkH*3, cmkY-cmkH ),
		wxPoint( cmkX+cmkH*3, cmkY-cmkH*2 ), wxPoint( cmkX,      cmkY-cmkH*2 )
	};

	mMemDC.DrawPolygon(cmbN, mkPts2, 0, 0);

	#define cmkSc 8
	static wxPoint mkPts3[cmbN] = {
		wxPoint( cmkX,        cmkY+cmkH*2+cmkSc ), wxPoint( cmkX+cmkH*3, cmkY+cmkH*2+cmkSc ),
		wxPoint( cmkX+cmkH*3, cmkY+cmkH*3+cmkSc ), wxPoint( cmkX,        cmkY+cmkH*3+cmkSc )
	};

	#define cOnBkColor  0x70,0xFF,0x70
	#define cOffBkColor 0xA0,0xA0,0xA0
	
	wxBrush mbBrushOn;
	wxBrush mbBrushOff;
	mbBrushOn.SetColour(cOnBkColor);
	mbBrushOff.SetColour(cOffBkColor);

	if(mKeyArray.Key[7]) {
		mMemDC.SetBrush(mbBrushOn);
	} else { mMemDC.SetBrush(mbBrushOff); }

	mMemDC.DrawPolygon(cmbN, mkPts3, 0,0);

	wxFont TextFont;
	TextFont.SetFaceName(_T("Microsoft Sans Serif"));
	TextFont.SetPointSize(8);
	mMemDC.SetFont(TextFont);
	mMemDC.SetTextForeground(wxColour(0x00,0x00,0x00));

	mMemDC.DrawText(_T(" - /"), mkPts3[0].x+25, mkPts3[0].y+20);

	Pen.SetColour(0x40,0x40,0x40);
	mMemDC.SetPen(Pen);

	//  
	#define cmkS 4
	#define cmkHs (cmkH-cmkS*2)
	
	static wxPoint mbPtsSt[cmbN] = {
		wxPoint( cmkX+cmkS,         cmkY+cmkH+cmkS ),       wxPoint( cmkX+cmkHs+cmkS,   cmkY+cmkH+cmkS ),
		wxPoint( cmkX+cmkHs+cmkS,   cmkY+cmkHs+cmkH+cmkS ), wxPoint( cmkX+cmkS,   cmkY+cmkHs+cmkH+cmkS )
	};

	wxPoint mbPts[cmbN];
	memcpy(&mbPts, &mbPtsSt, sizeof(wxPoint)*cmbN);

	#define cmbbN 7
	static wxPoint mbbPts[cmbbN] = {
		wxPoint( 0, 0 ), wxPoint( cmkH, 0 ), wxPoint( 0, -cmkH ), wxPoint( cmkH, cmkH ),
		wxPoint(0, -cmkH*3), wxPoint(-cmkH, 0), wxPoint(-cmkH, 0)
	};

	static wxChar* CmdTitle[cmbbN]  = {
		_T(" "),_T("  "),_T(" "),_T(" "),_T(" "),_T("  "),_T(" ")
	};

	for(int i=0; i<cmbbN; i++) {
		for(int j=0; j<cmbN; j++) {
			mbPts[j].x += mbbPts[i].x; mbPts[j].y += mbbPts[i].y;
		}

		if(mKeyArray.Key[i]) {
			mMemDC.SetBrush(mbBrushOn);
		} else { mMemDC.SetBrush(mbBrushOff); }

		mMemDC.DrawPolygon(cmbN, mbPts, 0,0);

		mMemDC.DrawText(CmdTitle[i], mbPts[0].x+2, mbPts[0].y+17);
	}

	Refresh();
}

void CIndicator::SetKeyArray(TKeyArray* apKeyArray)
{
	if(memcmp(apKeyArray, &mKeyArray, sizeof(TKeyArray))) {
		mKeyArray = *apKeyArray;
		Draw();
	}
}

//=============================================================================================
//=============================== Class: CRobotControllerApp ==================================
//=============================================================================================

//	  
class CRobotControllerApp : public wxApp
{
public:
	CMainWnd *Frame;

    virtual bool OnInit();

};

IMPLEMENT_APP(CRobotControllerApp)

//  
bool CRobotControllerApp::OnInit()
{
	Frame = new CMainWnd();
	Frame->Show(true);
    SetTopWindow(Frame);

    return true;
}


// : "  "
//=============================================================================================
//===================================== Class: CMainWnd =======================================
//=============================================================================================

//  
enum {
    wxID_RobotController_Quit = wxID_EXIT,
    wxID_RobotController_About = wxID_ABOUT,
 	wxID_TIMER
};

BEGIN_EVENT_TABLE(CMainWnd, wxFrame)
	//	   
    EVT_MENU(wxID_RobotController_Quit, CMainWnd::OnQuit)
  	EVT_TIMER(wxID_TIMER, CMainWnd::OnTimer)
	EVT_CLOSE(CMainWnd::OnClose)
END_EVENT_TABLE()

//  
CMainWnd::CMainWnd() : wxFrame((wxFrame *)NULL, wxID_ANY, _T(" "),
	                   wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE &
					   ~(wxRESIZE_BORDER|wxMAXIMIZE_BOX)), m_Timer(this, wxID_TIMER)
{
	//    
	bool AlreadyRunning;
    ::CreateMutex( NULL, FALSE, _T("Robot_Controller_V1-073FA540-B11D-12D3-BC37-009097709574"));
    AlreadyRunning = (::GetLastError() == ERROR_ALREADY_EXISTS || 
                      ::GetLastError() == ERROR_ACCESS_DENIED);
	if (AlreadyRunning) {
		wxMessageBox(_T("    ."),
					 _T("  "), wxOK | wxICON_ERROR);

		exit(0);
	}

	//     COM 
	CP_COMPortInit();

	mhCOMPort = CP_OpenCOMPort(1, CBR_57600, ODDPARITY, 8, 1);

	if(!mhCOMPort) { exit(0); }

	//     
	mhCmdExchangeSection = CreateFileMapping(INVALID_HANDLE_VALUE,
			                  NULL, PAGE_READWRITE, 0, sizeof(TKeyArray), _T("RobotCmdExchange"));
	gpKeyArray = (TKeyArray*) MapViewOfFile(mhCmdExchangeSection,FILE_MAP_ALL_ACCESS,0,0,sizeof(TKeyArray));
	memset(gpKeyArray, 0, sizeof(TKeyArray));



	SetIcon(wxICON(RobotController));

    mpPanel = new wxPanel(this, wxID_ANY);

	wxBoxSizer* MainSizer = new wxBoxSizer(wxVERTICAL);

	mpIndicator = new CIndicator(this);

	MainSizer->Add(mpIndicator,0,wxGROW,0);
	MainSizer->Fit(this);

	mpPanel->SetSizer(MainSizer);

	mpIndicator->Draw();

	hook_keyboard = SetWindowsHookEx(WH_KEYBOARD_LL,&LowLevelKeyboardProc,GetModuleHandle(NULL),0);

	m_Timer.Start(50);
}

//  
CMainWnd::~CMainWnd() {

}


//   - ""
void CMainWnd::OnQuit(wxCommandEvent& WXUNUSED(event))
{
	wxCloseEvent event;
	OnClose(event);
}

//   
void CMainWnd::OnClose(wxCloseEvent& WXUNUSED(event))
{
	int Cmd = wxMessageBox(_T("       ?"),
						   _T("    "),wxYES_NO);
	if(Cmd == wxYES) {

		UnhookWindowsHookEx(hook_keyboard);

		CP_CloseCOMPort(mhCOMPort);

		//      
		UnmapViewOfFile(gpKeyArray);
		CloseHandle(mhCmdExchangeSection);

		exit(0);
	}
}

//  
void CMainWnd::OnTimer(wxTimerEvent& WXUNUSED(event))
{
	//   
	enum {
		cmLEFT = 0,     // 
		cmBACKWARDS,    // 
		cmFORWARD,      // 
		cmRIGHT,        // 
		cmTURRET_RIGHT, //  
		cmFIRE,         // 
		cmTURRET_LEFT,  //  
		cmPOWER         // / 
	};

/*
	for(int i=0; i<cKeyTotal; i++) {
		if(gpKeyArray->Key[i]) {
			switch(i) {
				case cmFORWARD:

					break;

				case cmBACKWARDS:

					break;

				case cmTURRET_LEFT:

					break;

				case cmTURRET_RIGHT:

					break;

				case cmFIRE:

					break;

				case cmPOWER:

					break;
			}
		}
	}
*/
	//      COM 
	CP_WriteCOMPort(mhCOMPort, gpKeyArray->Key, sizeof(TKeyArray));

	mpIndicator->SetKeyArray(gpKeyArray);
}